@@ -25,30 +25,40 @@ def _write_xds_property(context, value):
 		win.property_delete('XdndDirectSave0')
 
 def _read_xds_property(context, delete):
+	"""Returns a UTF-8 encoded, non-escaped, URI."""
 	win = context.source_window
 	retval = win.property_get('XdndDirectSave0', 'text/plain', delete)
 	if retval:
 		return retval[2]
 	return None
 	
-def image_for_type(type):
+def image_for_type(type, size=48, flags=0):
 	'Search <Choices> for a suitable icon. Returns a pixbuf, or None.'
 	from icon_theme import rox_theme
+	
 	media, subtype = type.split('/', 1)
+
 	path = choices.load('MIME-icons', media + '_' + subtype + '.png')
+	icon=None
 	if not path:
-		icon = 'mime-%s:%s' % (media, subtype)
+		icon_name = 'mime-%s:%s' % (media, subtype)
+
 		try:
-			path = rox_theme.lookup_icon(icon, 48)
+			path=rox_theme.lookup_icon(icon_name, size, flags)
 			if not path:
-				icon = 'mime-%s' % media
-				path = rox_theme.lookup_icon(icon, 48)
+				icon_name = 'mime-%s' % media
+				path = rox_theme.lookup_icon(icon_name, size)
+
 		except:
 			print "Error loading MIME icon"
+
 	if not path:
 		path = choices.load('MIME-icons', media + '.png')
 	if path:
-		return gdk.pixbuf_new_from_file(path)
+		if hasattr(gdk, 'pixbuf_new_from_file_at_size'):
+			return gdk.pixbuf_new_from_file_at_size(path, size, size)
+		else:
+			return gdk.pixbuf_new_from_file(path)
 	else:
 		return None
 
@@ -67,7 +77,7 @@ class AbortSave(rox.UserAbort):
 	it!)"""
 	def __init__(self, message):
 		self.message = message
-		rox.UserAbort(self, message)
+		rox.UserAbort.__init__(self, message)
 	
 	def show(self):
 		if self.message:
@@ -87,7 +97,9 @@ class Saveable:
 		"""When the data is safely saved somewhere this is called
 		with its new name. Mark your data as unmodified and update
 		the filename for next time. Saving to another application
-		won't call this method. Default method does nothing."""
+		won't call this method. Default method does nothing.
+		The URI may be in the form of a URI or a local path.
+		It is UTF-8, not escaped (% really means %)."""
 		pass
 
 	def save_to_stream(self, stream):
@@ -307,7 +319,7 @@ class SaveArea(g.VBox):
 					self.document.save_to_file(path)
 				finally:
 					self.set_sensitive(True)
-				self.set_uri(path)
+				self.set_uri(uri)
 				self.save_done()
 			except:
 				_report_save_error()
@@ -377,7 +389,7 @@ class SaveArea(g.VBox):
 		to_send = 'E'
 		uri = _read_xds_property(context, False)
 		if uri:
-			path = get_local_path(uri)
+			path = get_local_path(escape(uri))
 			if path:
 				if not self.confirm_new_path(path):
 					to_send = 'E'
@@ -407,11 +419,7 @@ class SaveArea(g.VBox):
 	
 		if to_send != 'E':
 			_write_xds_property(context, None)
-			path = get_local_path(uri)
-			if path:
-				self.set_uri(path)
-			else:
-				self.set_uri(uri)
+			self.set_uri(uri)
 		if self.data_sent:
 			self.save_done()
 	
@@ -423,6 +431,9 @@ class SaveArea(g.VBox):
 		Returns true to go ahead with the save."""
 		if not os.path.exists(path):
 			return True
+		if os.path.isdir(path):
+			rox.alert(_("'%s' already exists as a directory.") % path)
+			return False
 		if path == self.initial_uri:
 			if self.document.save_last_stat is None:
 				return True		# OK. Nothing to compare with.
@@ -447,11 +458,11 @@ class SaveArea(g.VBox):
 	
 	def set_uri(self, uri):
 		"""Data is safely saved somewhere. Update the document's URI and save_last_stat (for local saves).
-		Internal."""
-		path = get_local_path(uri)
+		URI is not escaped. Internal."""
+		path = get_local_path(escape(uri))
 		if path is not None:
 			self.document.save_last_stat = os.stat(path)	# Record for next time
-		self.document.set_uri(uri)
+		self.document.set_uri(path or uri)
 	
 	def drag_end(self, widget, context):
 		self.drag_in_progress = 0
